/*
Ordinal Response Modeling with the LOGISTIC Procedure

This material is provided "as is" by SAS Institute Inc. There are no warranties, expressed or implied,
as to merchantability or fitness for a particular purpose regarding the materials or code contained herein.
The Institute is not responsible for errors in this material as it now exists or will exist, nor does the
Institute provide technical support for it.

https://support.sas.com/rnd/app/stat/papers/aastaffcode2013.html


*/

* LRTest Macro;

%macro LRTest(data=dsname,model1=,model2=,class=,freq=,print=1);                                            

/*Computes the likelihood ratio test between the two models.
  Small p-values mean the extra parameters in the larger model significantly
  describe some variation in the data.  Called by the %LRTestCycle macro.

  data=   name of the input data set. 
  model1= the MODEL statement for the first model without the word 'MODEL'. 
  model2= the MODEL statement for the 2nd model. 
  class=  the CLASS statement without the word 'CLASS'. 
  freq=   name of the FREQ variable, if there is one 
  print=  1=print the test results
*/ 

ods select globaltests;  
ods output globaltests=temp1; 

proc logistic data=&data; 
  %if (%length(&class)>0) %then class &class;;                                     
  model &model1; 
  %if (%length(&freq)>0) %then freq &freq;;                                     
RUN;

ods select globaltests; 
ods output globaltests=temp2; 

proc logistic data=&data; 
  %if (%length(&class)>0) %then class &class;;                                     
   model &model2; 
   %if (%length(&freq)>0) %then freq &freq;;                                     
RUN;

data temp2; set temp2;
rename ChiSq=ChiSq2 DF=DF2;
run; 

OPTIONS mergenoby=warn;

data temp; 
merge temp2 temp1; 
where Test='Likelihood Ratio'; 
keep Chisq DF P; 
ChiSq=abs(ChiSq2-ChiSq); 
DF= abs(DF2-DF); 
if (DF^=0) then P=1-probchi(ChiSq,DF);  
else P=.; 
run; 

OPTIONS mergenoby=error;

%if %eval(&print eq 1) %then
%do; proc print data=temp noobs; 
     VAR DF ChiSq P; 
     FORMAT p pval. ;
     title "&data: LRT: Model 2 vs Model 1";
     run; 
%end; 

%dld(temp2 temp1)

%mend; 
   

*LRTestCycle Macro;

%macro LRTestCycle(type,data=dsname,model=,class=,freq=);
/*
  Wraps %LRTest to compute certain tests for every effect. Small p-values mean
  the extra parameters in the larger model describe some variation in the data.

   data= name of the input data set.
  model= the MODEL statement without the word 'MODEL' and without unequalslopes.
         You must have spaces around the equal sign.
  class= the CLASS statement without the word 'CLASS'.
  freq = name of the FREQ variable, if there is one

         Models generated for the comparisons.
  type=  (AllX=all effects, OneX=one effect)
  ------ -------------------------------------
  Up     modelc1= Y = AllX / unequalslopes=OneX
         modelc2= Y = AllX
  Down   modelc1= Y = AllX / unequalslopes
         modelc2= Y = AllX / unequal=(not OneX)
  OneUp  modelc1= Y = OneX / unequalslopes=OneX
         modelc2= Y = OneX
*/

%let pos=%index(&model,=);
%if (&pos=0) %then %goto stopit2;
%let covars=%substr(&model,&pos+1);
%if (&covars=) %then %goto stopit2;
%let num=1;
%let var=%scan(&covars,&num);
%if (&var=) %then %goto stopit2;
%let pos=%index(&model,&var);
%do %while(&var ne);
   %if %eval(&type eq Up) %then %do;
      %let modelc1= &model / unequalslopes=&var;
      %let modelc2= &model;
   %end;
   %else %if %eval(&type eq Down) %then %do;
      %let modelc1= &model / unequalslopes;
      %let pos2=%index(&covars,&var);
      %let totlen=%length(&covars);
      %let len2= %eval(&pos2+%length(&var));
      %if (%eval(&pos2=1)) %then %let
         modelt2= &modelc1=(;
      %else %let
         modelt2= &modelc1=(%substr(&covars,1,&pos2-1);
      %if (&len2 <= &totlen) %then %let
         modelc2= &modelt2 %substr(&covars,&pos2+%length(&var)));
      %else %let
         modelc2= &modelt2);
   %end;
   %else %if %eval(&type = OneUp) %then %do;
      %let modelc1=%substr(&model,1,&pos-1) &var;
      %let modelc2=&modelc1 /unequalslopes=&var;
   %end;

%LRTest(data=&data,model1=&modelc1,model2=&modelc2, class=&class,freq=&freq,print=0);

data temp;
  set temp;
  length Var $10; *$;
  Var="&var";
run;
data tempall;
      set %if %eval(&num ne 1) %then tempall; temp;
run;

%let num=%eval(&num+1);
%let var=%scan(&covars,&num);
%if (&var=) %then %goto stopit2;
%end;

%stopit2: ;

proc print data=tempall label noobs;
   var Var DF ChiSq P;
run;

%mend;

   
* OneClassPlot Macro;

%macro OneClassPlot(data=dsname,x=x,y=y,numy=3,const=0,plot=1); 
/*
  A macro version of SAS Usage Note 37944 for CLASS covariates:   https://support.sas.com/kb/37/944.html
  Plots the empirical logits of Y against the levels of one CLASS variable X
  Called by the %EmpiricalLogitPlot macro

   data=  name of the input data set. 
      x=  name of the single CLASS covariate. 
      y=  name of the response variable.   
   numy=  number of levels of the response. 
  const=  small value in case of zero cells. 
   plot=  1=create the plot 
*/ 

ods graphics / antialiasmax=100000; 

%let numym1=%eval(&numy-1); 

proc freq data=&data noprint;    table &x*&y / out=temp2;
proc sort data=temp2;   by &x;                                                      
proc transpose data=temp2 out=temp; by &x;  var count; 

data temp2 ;  
   set temp; 
   length group $32;                                            
   if (_NAME_='COUNT');                                          
   group="&x"; 
   x=&x;         
   const=%sysevalf(&const+0); 
   %do i=1 %to &numym1; %let j=%eval(&i+1); 
      c&i=log((sum(of col1-col&i)+const) / (sum(of col&j-col&numy)+const)); 
      label c&i="Pr(&y <= &i)"; 
   %end; 
   keep c1-c&numym1 group x;                                              
run; 
%if %eval(&plot eq 1) %then %do; 

ODS graphics on / reset = all width= 4 in height=4 in border= off ;
ODS LISTING image_dpi=200 gpath="&pth" ;
ODS GRAPHICS / imagename = "one_class: &dsn" ;*IMAGEfmt=jpg;

proc sgplot; 
 %do i=1 %to &numym1;  
   series y=c&i x=x/ lineattrs=(thickness=1) markerattrs=(symbol=circlefilled size=5);  
 %end; 
   keylegend / noborder across = 3 down=3 valueattrs=(weight=bold size=6);
   yaxis values=(-5 to 5) label="Cumulative Logit" labelattrs=(weight=bold size=7) valueattrs=(weight=bold size=6); 
   xaxis offsetmin=.04 offsetmax=.04 integer label="&x" labelattrs=(weight=bold size=7) valueattrs=(weight=bold size=6); 
*title "one class plot: &dsname - &x";
run; 

%end; 
%mend; 

   
*OneContPlot Macro;

%macro OneContPlot(data=dsname,x=x,y=y,numy=3,const=0.5,nbhd=50,plot=1); 
/*
A macro extension of SAS Usage Note 37944 to continuous covariates.
Plots the empirical logits defined using the NBHD/2 observed X values preceding
and following each observation against the levels of a single continuous
var, then smooths the result.  Called by the %EmpiricalLogitPlot macro.

   data = name of the input data set
      x = name of the single continuous covariate 
      y = name of the response variable: y=1,2,3...
   numy = number of levels of the response
  const = small value in case of zero cells
   nbhd = total number of neighbors
   plot = 1=create the plot 
*/ 


data temp;  
   set &data; 
proc sort data=temp;  
   by &x &y;  
%let Full=%eval(&nbhd); 
%let Fullp1=%eval(&nbhd+1); 
%let Half=%eval(&nbhd/2); 
%let numym1=%eval(&numy-1); 
data temp2;  
   set temp; 
   retain %do i=1 %to &numy; sum&i %end; 0; 
   lagx= lag&Full(&x);  
   lagHalfx=lag&Half(&x);  
   lagy=lag&Fullp1(&y); 
   %do i=1 %to &numy; sum&i=sum&i+(&y=&i); %end; 
   if (lagy^=.) then do; 
      %do i=1 %to &numy; sum&i=sum&i-(lagy=&i); %end; 
   end; 
data temp; 
   set temp2;  
   if (_n_<&Full) then delete; 
data temp2;  
   set temp; 
   LENGTH group $32;
   const=%sysevalf(&const+0); 
   x=lagHalfx; 
   group="&x"; 
   %do i=1 %to &numym1; %let j=%eval(&i+1); 
   d&i=log((sum(of sum1-sum&i)+const) / (sum(of sum&j-sum&numy)+const)); 
   %end; 
   keep d1-d&numym1 group x;                                              
run; 
%if %eval(&plot eq 1) %then
%do;
   ods graphics / antialiasmax=100000; 
   proc sgplot; 
    %do i=1 %to &numym1;  
       loess y=d&i x=x / smooth=0.3 legendlabel="Pr(&y<=&i)" 
                         lineattrs=GraphData&i(thickness=1) 
                         markerattrs=(color=gray); 
    %end;
   yaxis values=(-5 to 5) label="Cumulative Probability"; 
   keylegend %do i=1 %to &numym1; "series&i" %end;;    
   run; 
%end; 

%mend; 

   
*EmpiricalLogitPlot Macro;

%macro EmpiricalLogitPlot(class=,cont=,data=dsname,y=response,numy=3,const=0.5); 

/*
  A macro extension of SAS Usage Note 37944 to class and continuous covariates.
  Plots the empirical logits defined using the NBHD/2 observed X values preceding
  and following each observation against the levels of a single continuous covariate,
  and plots the empirical logits against the levels of a single CLASS covariate X.
  Calls the %OneClassPlot and %OneContPlot macros.
  
  class= space-separated list of CLASS effects. 
   cont= space-separated list of continuous efffects. 
   data= name of the input data set. 
      y= name of the response variable. y=1,2,3... 
   numy= number of levels of the response variable. 
  const= small value in case of zero cells. 
*/ 

%let num=1; 
%let numc=0; 
%let numd=0; 
%let totnum=1; 
%let doClass=1; 
%let numym1=%eval(&numy-1); 
%let var=%scan(&class,&num);                                  
%if (&var=) %then
%do; %let doClass=0;
     %let var=%scan(&cont,&num);
     %if (&var=) %then %goto stopit;
%end;                                
%do %while(&var ne);                                             
   data tempa(keep=&y &var);                                                          
      set &data;                                                 
   %if %eval(&doClass eq 1) %then %do; 
      %OneClassPlot(data=tempa,x=&var,y=&y,numy=&numy,const=&const,plot=0); 
      %let numc= %eval(&numc+1); 
      data tempc; 
         set %if %eval(&numc ne 1) %then tempc; temp2; 
      run; 
   %end; 
   %else %do; 
      %OneContPlot(data=tempa,x=&var,y=&y,numy=&numy,const=&const,plot=0); 
      %let numd= %eval(&numd+1); 
      data tempd; 
         set %if %eval(&numd ne 1) %then tempd; temp2; 
      run; 
   %end; 
   %let num=%eval(&num+1); 
   %let totnum=%eval(&totnum+1);; 
   %if %eval(&doClass eq 1) %then %do;                                          
      %let var=%scan(&class,&num); 
      %if (&var=) %then %do; 
          %let doClass=0; 
          %let num=1; 
      %end;
   %end; 
   %if %eval(&doClass eq 0) %then %do;            
      %let var=%scan(&cont,&num); 
      %if (&var=) %then %goto stopit; 
   %end;
%end;
%stopit: ; 
%if %eval(&numc>0) %then %do;
proc sort data=tempc; 
   by group x;
%end;
%if %eval(&numd>0) %then %do;
proc sort data=tempd; 
   by group x;
%end;
%if %eval(&numc>0 and &numd>0) %then %do;
data tempall;
   merge tempd tempc;
   by group x;
%end;
%else %if %eval(&numc>0) %then %do;
data tempall;
   set tempc;
%end;
%else %do;
data tempall;
   set tempd;
%end;
proc sort data=tempall;  
   by group; 
data tempa(keep=group minx maxx);  
   set tempall;  
   by group;  
   retain minx 1e100 maxx -1e100; 
   if first.group then do;  
      minx=x;  
      maxx=x;  
   end; 
   minx=min(minx,x);  
   maxx=max(maxx,x); 
   if last.group then output;  
DATA tempall;  
merge tempall tempa;  
by group;  
x=(x-minx)/(maxx-minx);
RUN;

*proc print data=tempall; *run;


%if       ((&totnum-1)=1)   %then %do; %let rows=1; %let cols=1; %let hgt = 3; %let wdt = 4; %end;    
%else %if ((&totnum-1)=2)   %then %do; %let rows=1; %let cols=2; %let hgt = 3; %let wdt = 5; %end;    
%else %if ((&totnum-1)=3)   %then %do; %let rows=1; %let cols=3; %let hgt = 3; %let wdt = 7; %end;    
%else %if ((&totnum-1)=4)   %then %do; %let rows=2; %let cols=2; %let hgt = 5; %let wdt = 5; %end;    
%else %if ((&totnum-1)<=6)  %then %do; %let rows=2; %let cols=3; %let hgt = 5; %let wdt = 7; %end;    
%else %if ((&totnum-1)<=9)  %then %do; %let rows=3; %let cols=3; %let hgt = 7; %let wdt = 7; %end;    
%else %if ((&totnum-1)<=12) %then %do; %let rows=3; %let cols=4; %let hgt = 7; %let wdt = 9; %end;    
%else %do;       %let rows=ceil((&totnum-1)/&cols); %let cols=5; %let hgt =10; %let wdt =10; %end;    

ODS graphics on / reset = all width= &wdt. in height=&hgt. in border= off ;
ODS graphics / loessmaxobs=10000 antialiasmax=100000; 
ODS LISTING image_dpi=200 gpath="&pth" ;
ODS GRAPHICS / imagename = "ELP_&y._&data." ;*IMAGEfmt=jpg;

proc sgpanel data=tempall;                                           
panelby group / novarname columns=&cols. rows=&rows. uniscale=all headerattrs=(weight=bold size=7);
rowaxis offsetmin=.025 offsetmax=.025 label='Empirical Cumulative Logits' labelattrs=(weight=bold size=9); 
colaxis offsetmin=.025 offsetmax=.025 display=(nolabel noticks novalues);                     

%if (&numd > 0) %then %do i=1 %to &numym1;
   loess   y=d&i x=x / smooth=0.3 legendlabel="&y<=&i" name="series&i" lineattrs=GraphData&i(thickness=1) markerattrs=GraphDataDefault(color=white size=0) ; 
   * scatter y=d&i x=x / jitter markerattrs=(symbol=circlefilled color=black size=2) ;
%end; 
%if (&numc > 0) %then %do i=1 %to &numym1; 
   series y=c&i x=x / legendlabel="&y<=&i" name="series&i"  lineattrs=GraphData&i(thickness=1 ); 
%end;    
keylegend %do i=1 %to &numym1; "series&i" %end; / noborder across=2 down=2 valueattrs=(weight=bold size=8) linelength=45;
title "Empirical Cumulative Logit Plots";
run;

%ogf

%dld(temp temp2 tempa tempc tempd tempall );

%mend;

   
*EXPlot Macro;

%macro EXPlot(data=dsname,y=response,numy=3,const=0.5) / parmbuff; 

/*
  Compares the mean of the X variable within each level of Y, to the expected value of X|Y=j
  given that the proportional odds assumption holds, E(X|Y=j).   From Harrell (2001).

  Syntax:
  %EXPlot(effect-list,data=,y=,numy=,const=); 

  effect-list space-separated list of all effects (covariates) to plot 
     data = name of the input data set
        y = name of the response variable  Y=1,2,3,...,numy 
     numy = number of levels of the response
    const = small value in case of zero cells
*/ 

%let num=1; 
%let numym1=%eval(&numy-1);                                                       
%let var=%scan(&syspbuff,&num);                                   
%if (%index(&var,=)) %then %goto stopit3;                                 

data temp;   set &data;
proc sort data=temp;  by &y;

%do %while(&var ne);                                                                      
   proc means data=temp noprint;                                    
      var &var;                                                      
      by &y;                                                         
      output out=tempc mean=mean max=max;                                

    data tempc(keep=&y mean);  
      set tempc;  
      retain maxx -1e100;  
      maxx= MAX(maxx,max);                                           
      call symputx('scale',maxx);                                     
     RUN;

proc freq data=temp noprint;                                     
      tables &y / out=tempb;                                               
*proc print data=tempb; *title 'y values'; 
RUN;

data _null_;
set tempb;
call symputx('count'||left(_n_),COUNT);

proc logistic data=temp noprint;                                         
model &y =&var;                                                
output out=tempout predprobs=(i);                                  

proc sort data=tempout; by &var;

data tempa(keep=exy1-exy&numy);  
      set tempout end=eof;
      array exy exy1-exy&numy (%do i=1 %to &numy; 0 %end;); 
      %do i=1 %to &numy;
         if (ip_&i^=.) then exy[&i]=exy[&i]+ ip_&i*&var/&&count&i; 
      %end;
      if (eof) then output;                                          
proc transpose data=tempa out=tempd prefix=exp;

OPTIONS mergenoby=warn;
DATA tempa;
MERGE tempb tempc tempd;
length group $32;
group="&var";
exp1=exp1/&scale;  
mean=mean/&scale;  
RUN;

 DATA tempall; 
   SET %if %eval(&num ne 1) %then tempall; tempa; 
 RUN; 
 %let num=%eval(&num+1);                                           
 %let var=%scan(&syspbuff,&num);                                   
 %if (%index(&var,=)) %then %goto stopit3;                                 

%end; 
%stopit3: ;

proc sort data=tempall; 
by group;                                     

data tempa(keep=group miny maxy);  
set tempall;  
by group; 
retain maxy -1e100 miny 1e100;         
if first.group then
   do; maxy=max(exp1,mean);  
       miny=min(exp1,mean);  
   end;                                            
maxy=max(maxy,exp1,mean);  
miny=min(miny,exp1,mean);               
if last.group then output;                                        

DATA tempall;  
MERGE tempall tempa;  
BY group;
label exp1="E(X|&y)"
      mean="Mean";
exp1=(exp1-miny)/(maxy-miny);
mean=(mean-miny)/(maxy-miny);
RUN;

   *OPTIONS mergenoby=error;

   %if       ((&num-1) = 1) %then %do; %let rows=1; %let cols=1; %let hgt = 3; %let wdt = 4; %end;    
   %else %if ((&num-1) = 2) %then %do; %let rows=1; %let cols=2; %let hgt = 3; %let wdt = 5; %end;    
   %else %if ((&num-1) = 3) %then %do; %let rows=1; %let cols=3; %let hgt = 3; %let wdt = 7; %end;    
   %else %if ((&num-1) = 4) %then %do; %let rows=2; %let cols=2; %let hgt = 5; %let wdt = 5; %end;    
   %else %if ((&num-1)<= 6) %then %do; %let rows=2; %let cols=3; %let hgt = 5; %let wdt = 7; %end;    
   %else %if ((&num-1)<= 9) %then %do; %let rows=3; %let cols=3; %let hgt = 7; %let wdt = 7; %end;    
   %else %if ((&num-1)<=12) %then %do; %let rows=3; %let cols=4; %let hgt = 7; %let wdt = 9; %end;    
   %else %do;       %let rows=ceil((&num-1)/&cols); %let cols=5; %let hgt = 9; %let wdt =10; %end;    

proc print data=tempall; run;

ODS graphics on / reset = all width= &wdt. in height = &hgt in border= off ;
ODS graphics / antialiasmax=100000; 
ODS LISTING image_dpi=200 gpath="&pth" ;
ODS GRAPHICS / imagename = "EXP_&y._&data." ;*IMAGEfmt=jpg;

proc sgpanel data=tempall;                                            
   panelby group / novarname columns=&cols rows=&rows uniscale=column headerattrs=(weight=bold size=7);    
   rowaxis display=(nolabel);
   colaxis integer;
   series y=exp1 x=&y / lineattrs=(thickness=2 px) name='ex'; 
   series y=mean x=&y / lineattrs=(thickness=2 px) name='mn'; 
KEYLEGEND 'ex' 'mn' / noborder across=2 down=2 titleattrs=(weight=bold size=10) valueattrs=(weight=bold size=9) ;
COLAXIS offsetmin=.04 offsetmax=.04 values =(1 to &numy. by 1) valueattrs=(weight=bold size=6)
        label ='Response' labelattrs=(weight=bold size=9);
ROWAXIS offsetmin=.025 offsetmax=.025 labelattrs=(weight=bold size=7) valueattrs=(weight=bold size=6);
TITLE1 "&data.: Ratio of mean of X within each level of Y,";
TITLE2 "to the expected value of X|Y=j";
run;                                                              

%ogf

%dld(temp tempa tempb tempc tempd tempout tempall )

%mend;

